@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2010-2012, 2014-2017 Free Software Foundation, Inc.
+   Copyright (C) 2010-2021 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
@@ -21,13 +21,18 @@
 # include <mailutils/stream.h>
 # include <mailutils/sys/stream.h>
 
+struct _mu_xscript_channel
+{
+  int state;
+  size_t length;
+};
+
 struct _mu_xscript_stream
 {
   struct _mu_stream stream;
   mu_stream_t transport;
   mu_stream_t logstr;
-  int flags;
-  int level;
+  struct _mu_xscript_channel channel[2];
   char *prefix[2];
 };
 
