@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001-2003, 2005, 2007, 2010-2012, 2014-2017 Free
-   Software Foundation, Inc.
+   Copyright (C) 1999-2021 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -28,7 +27,8 @@ mail_delete_msg (msgset_t *mspec, mu_message_t msg, void *data)
 
   mu_message_get_attribute (msg, &attr);
   mu_attribute_set_deleted (attr);
-  cond_page_invalidate (mspec->msg_part[0]);
+  mu_attribute_unset_userflag (attr, MAIL_ATTRIBUTE_PRESERVED);
+  cond_page_invalidate (msgset_msgno (mspec));
   return 0;
 }
 
@@ -38,7 +38,7 @@ mail_delete (int argc, char **argv)
   int rc = util_foreach_msg (argc, argv, MSG_NODELETED|MSG_SILENT,
 			     mail_delete_msg, NULL);
 
-  if (mailvar_is_true ("autoprint"))
+  if (mailvar_is_true (mailvar_name_autoprint))
     util_do_command("print");
 
   return rc;
