@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2019 Free Software Foundation, Inc.
+   Copyright (C) 1999-2021 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -150,6 +150,8 @@ cli_attach (struct mu_parseopt *po, struct mu_option *opt, char const *arg)
 			default_content_type, default_encoding))
     exit (po->po_exit_error);
 
+  mime_option = 1;
+  
   free (content_name);
   content_name = NULL;
   free (content_filename);
@@ -260,9 +262,11 @@ static struct mu_option mail_options[] = {
     mu_c_string, &default_content_type },
 
   { "content-name", 0, N_("NAME"), MU_OPTION_DEFAULT,
+    /* TRANSLATORS: Don't translate "Content-Type" and "name"! */
     N_("set the Content-Type name parameter for the next --attach option"),
     mu_c_string, &content_name },
   { "content-filename", 0, N_("NAME"), MU_OPTION_DEFAULT,
+    /* TRANSLATORS: Don't translate "Content-Disposition" and "filename"! */
     N_("set the Content-Disposition filename parameter for the next --attach option"),
     mu_c_string, &content_filename },
 
@@ -291,6 +295,7 @@ static const char *alt_args[] = {
 static struct mu_cli_setup cli = {
   options,
   NULL,
+  /* TRANSLATORS: "mail" is the name of the program. Don't translate it. */
   N_("GNU mail -- process mail messages.\n"
      "If -f or --file is given, mail operates on the mailbox named "
      "by the first argument, or the user's mbox, if no argument given."),
@@ -317,7 +322,8 @@ mail_cmdline (void *closure, int cont MU_ARG_UNUSED)
 
   while (1)
     {
-      if (mailvar_is_true ("autoinc") && !mu_mailbox_is_updated (mbox))
+      if (mailvar_is_true (mailvar_name_autoinc)
+	  && !mu_mailbox_is_updated (mbox))
 	{
 	  mu_mailbox_messages_count (mbox, &total);
 	  page_invalidate (0);
@@ -332,7 +338,7 @@ mail_cmdline (void *closure, int cont MU_ARG_UNUSED)
 	  continue;
 	}
 
-      if (!rc && mailvar_is_true ("ignoreeof"))
+      if (!rc && mailvar_is_true (mailvar_name_ignoreeof))
 	{
 	  mu_error (_("Use \"quit\" to quit."));
 	  continue;
@@ -480,7 +486,7 @@ main (int argc, char **argv)
     util_do_command ("set columns=%d", util_getcols ());
 
   /* Set the default mailer to sendmail.  */
-  mailvar_set ("sendmail",
+  mailvar_set (mailvar_name_sendmail,
 	       mu_strdup ("sendmail:" PATH_SENDMAIL), mailvar_type_string,
 	       MOPTF_OVERWRITE);
 
@@ -535,7 +541,7 @@ main (int argc, char **argv)
 
 
   /* read system-wide mail.rc and user's .mailrc */
-  if (mailvar_is_true ("rc"))
+  if (mailvar_is_true (mailvar_name_rc))
     util_do_command ("source %s", SITE_MAIL_RC);
   if ((p = getenv ("MAILRC")) && *p)
     util_do_command ("source %s", getenv ("MAILRC"));
@@ -564,7 +570,7 @@ main (int argc, char **argv)
     }
 
   /* how should we be running? */
-  if (mailvar_get (&mode, "mode", mailvar_type_string, 1))
+  if (mailvar_get (&mode, mailvar_name_mode, mailvar_type_string, 1))
     exit (EXIT_FAILURE);
 
   /* Interactive mode */
@@ -580,7 +586,8 @@ main (int argc, char **argv)
 
       mu_argcv_string (argc, argv, &buf);
       rc = util_do_command ("mail %s", buf);
-      return mailvar_is_true ("mailx") ? 0 : rc;
+      free (buf);
+      return mailvar_is_true (mailvar_name_mailx) ? 0 : rc;
     }
   /* Or acting as a normal reader */
   else
@@ -618,11 +625,21 @@ main (int argc, char **argv)
 
       if ((rc = mu_mailbox_open (mbox, MU_STREAM_RDWR|MU_STREAM_CREAT)) != 0)
 	{
-	  mu_url_t url = NULL;
-	  mu_mailbox_get_url (mbox, &url);
-	  mu_error (_("Cannot open mailbox %s: %s"),
-		      mu_url_to_string (url), mu_strerror (rc));
-	  mu_mailbox_destroy (&mbox);
+	  if (rc == EACCES)
+	    {
+	      rc = mu_mailbox_open (mbox, MU_STREAM_READ);
+	      if (rc == 0)
+		mu_diag_output (MU_DIAG_WARNING, _("mailbox opened read-only"));
+	      //FIXME: Disable 'q' and similar commands
+	    }
+	  if (rc)
+	    {
+	      mu_url_t url = NULL;
+	      mu_mailbox_get_url (mbox, &url);
+	      mu_error (_("Cannot open mailbox %s: %s"),
+			mu_url_to_string (url), mu_strerror (rc));
+	      mu_mailbox_destroy (&mbox);
+	    }
 	}
 
       if (rc)
@@ -650,13 +667,14 @@ main (int argc, char **argv)
 	  else if (strcmp (mode, "read"))
 	    {
 	      mu_error (_("Unknown mode `%s'"), mode);
-	      util_do_command ("quit");
+	      util_do_command (mailvar_name_quit);
 	      return 1;
 	    }
 	}
 
       if (total == 0
-	  && (strcmp (mode, "read") || !mailvar_is_true ("emptystart")))
+	  && (strcmp (mode, "read")
+	      || !mailvar_is_true (mailvar_name_emptystart)))
 	{
 	  if (secondary_url)
 	    mail_summary (0, NULL);
@@ -666,16 +684,16 @@ main (int argc, char **argv)
 	}
 
       /* initial commands */
-      if (mailvar_is_true ("header"))
+      if (mailvar_is_true (mailvar_name_header))
 	{
 	  util_do_command ("summary");
 	  util_do_command ("headers");
 	}
 
-      mailvar_get (&prompt, "prompt", mailvar_type_string, 0);
+      mailvar_get (&prompt, mailvar_name_prompt, mailvar_type_string, 0);
       mail_mainloop (mail_cmdline, (void*) prompt, 1);
       mu_printf ("\n");
-      util_do_command ("quit");
+      util_do_command (mailvar_name_quit);
       return 0;
     }
   /* We should never reach this point */
@@ -716,33 +734,13 @@ mail_mainloop (char *(*input) (void *, int),
       if (do_history && !(mu_isspace (cmd[0]) || cmd[0] == '#'))
 	add_history (cmd);
 #endif
-      if (command)
-	free (command);
+      free (command);
     }
 }
 
 int
 mail_warranty (int argc MU_ARG_UNUSED, char **argv MU_ARG_UNUSED)
 {
-  mu_printf (
-	 _("GNU Mailutils -- a suite of utilities for electronic mail\n"
-	   "Copyright (C) 1999-2019 Free Software Foundation, Inc.\n\n"));
-  mu_printf (
-  _("   GNU Mailutils is free software; you can redistribute it and/or modify\n"
-    "   it under the terms of the GNU General Public License as published by\n"
-    "   the Free Software Foundation; either version 3 of the License, or\n"
-    "   (at your option) any later version.\n"
-    "\n"
-    "   GNU Mailutils is distributed in the hope that it will be useful,\n"
-    "   but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
-    "   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
-    "   GNU General Public License for more details.\n"
-    "\n"
-    "   You should have received a copy of the GNU General Public License along\n"
-    "   with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>.\n"
-    "\n"
-    "\n"
-));
-
+  mu_version_print (mu_strout);
   return 0;
 }
