@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001, 2005, 2007, 2009-2012, 2014-2018 Free
-   Software Foundation, Inc.
+   Copyright (C) 1999-2021 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -30,10 +29,10 @@ size0 (msgset_t *mspec, mu_message_t msg, void *data)
   mu_message_lines (msg, &lines);
   
   mu_printf ("%c%2lu %3lu/%-5lu\n",
-		    is_current_message (mspec->msg_part[0]) ? '>' : ' ',
-		    (unsigned long) mspec->msg_part[0],
-		    (unsigned long) lines,
-		    (unsigned long) size);
+	     is_current_message (msgset_msgno (mspec)) ? '>' : ' ',
+	     (unsigned long) msgset_msgno (mspec),
+	     (unsigned long) lines,
+	     (unsigned long) size);
   return 0;
 }
 
